@@ -1,4 +1,4 @@
-#     Copyright 2024, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
+#     Copyright 2025, Kay Hayen, mailto:kay.hayen@gmail.com find license text at end of file
 
 
 """
@@ -33,6 +33,7 @@ from nuitka.PythonVersions import python_version
 from nuitka.Tracing import plugins_logger, printLine
 from nuitka.utils.FileOperations import (
     getDllBasename,
+    getNormalizedPath,
     makePath,
     putTextFileContents,
 )
@@ -50,7 +51,7 @@ from .PluginBase import NuitkaPluginBase, control_tags
 # Maps plugin name to plugin instances.
 active_plugins = OrderedDict()
 plugin_name2plugin_classes = {}
-plugin_options = {}
+plugin_options = OrderedDict()
 plugin_values = {}
 user_plugins = OrderedSet()
 
@@ -259,6 +260,16 @@ Error, unknown plug-in '%s' in wrong case referenced, use '%s' instead."""
     return plugin_name2plugin_classes[plugin_name][0]
 
 
+def hasPluginName(plugin_name):
+    # First, load plugin classes, to know what we are talking about.
+    loadPlugins()
+
+    # Backward compatibility.
+    plugin_name = Options.getPluginNameConsideringRenames(plugin_name)
+
+    return plugin_name in plugin_name2plugin_classes
+
+
 def _addPluginClass(plugin_class, detector):
     plugin_name = plugin_class.plugin_name
 
@@ -403,7 +414,7 @@ class Plugins(object):
                 if not checkModuleName(v):
                     plugin.sysexit(
                         "Plugin '%s' returned an invalid module name, not %r (for module '%s')"
-                        % (plugin, v, module.getFullName())
+                        % (plugin.plugin_name, v, module.getFullName())
                     )
 
                 yield ModuleName(v)
@@ -592,7 +603,7 @@ class Plugins(object):
     def onOnefileFinished(filename):
         """Let plugins post-process the onefile executable in onefile mode"""
         for plugin in getActivePlugins():
-            plugin.onStandaloneDistributionFinished(filename)
+            plugin.onOnefileFinished(filename)
 
     @staticmethod
     def onBootstrapBinary(filename):
@@ -784,9 +795,11 @@ class Plugins(object):
 
         try:
             trigger_module = buildModule(
-                module_filename=os.path.join(
-                    os.path.dirname(module.getCompileTimeFilename()),
-                    module_name.asPath() + ".py",
+                module_filename=getNormalizedPath(
+                    os.path.join(
+                        os.path.dirname(module.getCompileTimeFilename()),
+                        module_name.asPath() + ".py",
+                    )
                 ),
                 module_name=module_name,
                 reason="trigger",
@@ -1667,17 +1680,37 @@ def loadPlugins():
         loadStandardPluginClasses()
 
 
-def addStandardPluginCommandLineOptions(parser):
+# TODO: Use in more places.
+def _keyPluginForSort(plugin_class):
+    return plugin_class.getCategories(), plugin_class.plugin_name
+
+
+def _keyPluginForSort2(item):
+    _plugin_name, (plugin_class, _plugin_detector) = item
+
+    return _keyPluginForSort(plugin_class)
+
+
+def addStandardPluginCommandLineOptions(parser, plugin_help_mode):
     loadPlugins()
 
     for _plugin_name, (plugin_class, _plugin_detector) in sorted(
-        plugin_name2plugin_classes.items()
+        plugin_name2plugin_classes.items(), key=_keyPluginForSort2
     ):
         if plugin_class.isAlwaysEnabled():
-            _addPluginCommandLineOptions(
-                parser=parser,
-                plugin_class=plugin_class,
-            )
+            if not plugin_help_mode or not plugin_class.hasCategory("package-support"):
+                _addPluginCommandLineOptions(
+                    parser=parser,
+                    plugin_class=plugin_class,
+                    plugin_help_mode=plugin_help_mode,
+                )
+        else:
+            if plugin_help_mode:
+                _addPluginCommandLineOptions(
+                    parser=parser,
+                    plugin_class=plugin_class,
+                    plugin_help_mode=plugin_help_mode,
+                )
 
 
 def activatePlugins():
@@ -1759,11 +1792,27 @@ def activatePlugins():
             _addActivePlugin(plugin_detector, args=False)
 
 
-def _addPluginCommandLineOptions(parser, plugin_class):
+def _addPluginCommandLineOptions(parser, plugin_class, plugin_help_mode):
     plugin_name = plugin_class.plugin_name
 
     if plugin_name not in plugin_options:
-        option_group = OptionGroup(parser, "Plugin options of '%s'" % plugin_name)
+        plugin_display_name = repr(plugin_name)
+
+        if plugin_help_mode:
+            # Ignore duplicate per design givers.
+            if plugin_name in ("pyside2", "pyqt6", "pyqt5"):
+                return
+
+            if plugin_name == "pyside6":
+                plugin_display_name = (
+                    "'pyside6' (same for 'pyside2', 'pyqt6', 'pyqt5' plugins)"
+                )
+
+        option_group = OptionGroup(
+            parser,
+            "Plugin options of %s (categories: %s)"
+            % (plugin_display_name, ", ".join(plugin_class.getCategories())),
+        )
         try:
             plugin_class.addPluginCommandLineOptions(option_group)
         except OptionConflictError as e:
@@ -1786,7 +1835,7 @@ def _addPluginCommandLineOptions(parser, plugin_class):
             plugin_options[plugin_name] = ()
 
 
-def addPluginCommandLineOptions(parser, plugin_names):
+def addPluginCommandLineOptions(parser, plugin_names, plugin_help_mode):
     """Add option group for the plugin to the parser.
 
     Notes:
@@ -1800,12 +1849,19 @@ def addPluginCommandLineOptions(parser, plugin_names):
     """
     for plugin_name in plugin_names:
         plugin_class = getPluginClass(plugin_name)
-        _addPluginCommandLineOptions(parser=parser, plugin_class=plugin_class)
+        _addPluginCommandLineOptions(
+            parser=parser, plugin_class=plugin_class, plugin_help_mode=plugin_help_mode
+        )
 
 
 def addUserPluginCommandLineOptions(parser, filename):
     plugin_class = loadUserPlugin(filename)
-    _addPluginCommandLineOptions(parser=parser, plugin_class=plugin_class)
+    _addPluginCommandLineOptions(
+        parser=parser,
+        plugin_class=plugin_class,
+        # List those user plugins always.
+        plugin_help_mode=True,
+    )
 
     user_plugins.add(plugin_class)
 
